@@ -26,7 +26,6 @@ import subprocess
 import sys
 import tempfile
 
-from oslo_utils import importutils
 from oslo_utils import strutils
 
 from ironicclient.common.i18n import _
@@ -73,13 +72,6 @@ def define_commands_from_module(subparsers, command_module, cmd_mapper):
         define_command(subparsers, command, callback, cmd_mapper)
 
 
-def import_versioned_module(version, submodule=None):
-    module = 'ironicclient.v%s' % version
-    if submodule:
-        module = '.'.join((module, submodule))
-    return importutils.import_module(module)
-
-
 def split_and_deserialize(string):
     """Split and try to JSON deserialize a string.
 
@@ -101,11 +93,33 @@ def split_and_deserialize(string):
     return (key, value)
 
 
+def key_value_pairs_to_dict(key_value_pairs):
+    """Convert a list of key-value pairs to a dictionary.
+
+    :param key_value_pairs: a list of strings, each string is in the form
+                            <key>=<value>
+    :returns: a dictionary, possibly empty
+    """
+    if key_value_pairs:
+        return dict(split_and_deserialize(v) for v in key_value_pairs)
+    return {}
+
+
 def args_array_to_dict(kwargs, key_to_convert):
+    """Convert the value in a dictionary entry to a dictionary.
+
+    From the kwargs dictionary, converts the value of the key_to_convert
+    entry from a list of key-value pairs to a dictionary.
+
+    :param kwargs: a dictionary
+    :param key_to_convert: the key (in kwargs), whose value is expected to
+        be a list of key=value strings. This value will be converted to a
+        dictionary.
+    :returns: kwargs, the (modified) dictionary
+    """
     values_to_convert = kwargs.get(key_to_convert)
     if values_to_convert:
-        kwargs[key_to_convert] = dict(split_and_deserialize(v)
-                                      for v in values_to_convert)
+        kwargs[key_to_convert] = key_value_pairs_to_dict(values_to_convert)
     return kwargs
 
 
